@@ -183,44 +183,56 @@ class Quivers:
                 visiting[n] = False
                 return False
 
-    def transitiveClosure(self,inPlace=False):
+    def transitiveClosure(self,inPlace=False,lenient=False):
         '''
         Produces a Quivers that is transitively closed.
         If inPlace is True, this object is changed. Otherwise,
         a new Quivers object is returned.
+        If lenient is True, cycles will not lead to exceptions.
         '''
 
-        def _fillInDescendants(n, visited, visiting, data):
+        def _fillInDescendants(n, visited, visiting, visitseq, data):
             if visiting[n]:
-                raise Exception('Cycle in Quivers object detected %d -> %d' % (n,n))
+                if not lenient:
+                    raise Exception('Cycle in Quivers object detected %d -> %d' % (n,n))
+                else:
+                    i = visitseq.index(n)
+                    decs = set.union(*data[i:])
+                    decs.add(n)
+                    return decs
 
             if visited[n]:
                 return data[n]
 
             visiting[n] = True
-            result      = set()
+            visitseq.append(n)
+            decs        = data[n]
             for d in self.quivers[n]:
-                result.add(d)
+                decs.add(d)
                 if not visited[d]:
-                    result.update(_fillInDescendants(d,visited,visiting,data))
+                    decs.update(_fillInDescendants(d,visited,visiting,visitseq,data))
                 else:
-                    result.update(data[d])
+                    decs.update(data[d])
                 
             visited[n]  = True
             visiting[n] = False
-            data[n]     = sorted(result)
-            return result
-        
+            visitseq.pop()
+            #data[n]     = set(decs)
+            return decs
 
+        
         N        = len(self)
-        data     = [[]]*N
+        data     = [set() for _ in range(N)]
         visited  = [False]*N
         visiting = [False]*N
+        visitseq = []
         for i in range(N):
-            _fillInDescendants(i,visited,visiting,data)
+            _fillInDescendants(i,visited,visiting,visitseq,data)
 
+        
+            
         if inPlace:
-            self.quivers = data
+            self.quivers = [sorted(d) for d in data]
             self.transitivelyClosed = True
             return self
         else:
@@ -263,13 +275,13 @@ class Quivers:
         return [revMap.get(n,set()) for n in range(len(self))]
 
 
-    def degrees(self):
+    def degrees(self,transitive=False):
         '''
         Returns a sequence of pairs, one pair for every
         element, containing the elements' in- and out degree.
         '''
         Q = self
-        if not self.transitivelyClosed:
+        if transitive and not self.transitivelyClosed:
             Q = self.transitiveClosure(inPlace=False)
 
         outs = [len(x) for x in Q.quivers]
@@ -280,6 +292,16 @@ class Quivers:
                 
         return zip(ins,outs)
 
+    def toAdjacencyMatrix(self):
+        import numpy as np
+        n = len(self)
+        A = np.zeros((n,n), dtype=int)
+        for i in range(n):
+            for j in self.quivers[i]:
+                A[i,j] = 1
+
+        return A
+    
     def __getitem__(self,i):
         return self.quivers[i]
 
@@ -565,15 +587,16 @@ class BasicLinkage:
 
         return DistMatrix(dists=dists)
 
-
-def getLinkage(L):
+def getLinkageFactory(L):
     if L == 'single':
-        return BasicLinkage(SingleLinkage())
-    elif L == 'complete':
-        return BasicLinkage(CompleteLinkage())
-    else:
-        raise Error('Unknow linkage: "%d"' % L)
+        return lambda x : BasicLinkage(SingleLinkage())
+    if L == 'complete':
+        return lambda x : BasicLinkage(CompleteLinkage())
+    if L == 'average':
+        return lambda x : BasicLinkage(AverageLinkage(x))
 
+    raise Exception('Unknown linkage model: "%s"' % str(L))
+    
 def SingleLinkage():
     class SL:
         def __call__(self,i,j,x,M):
@@ -621,8 +644,8 @@ class AgglomerativeClustering:
         assert (joins is None) == (dists is None)
         if joins:
             assert len(joins) == len(dists)
-            self.joins = joins
-            self.dists = dists
+            self.joins = list(joins)
+            self.dists = list(dists)
         else:
             self.joins = []
             self.dists = []
